@@ -1,23 +1,20 @@
-// Copyright 2013-2015, The Gtk-rs Project Developers.
-// See the COPYRIGHT file at the top-level directory of this distribution.
-// Licensed under the MIT license, see the LICENSE file or <http://opensource.org/licenses/MIT>
+// Take a look at the license at the top of the repository in the LICENSE file.
 
+use crate::Cursor;
+use crate::EventMask;
+use crate::Visual;
+use crate::Window;
 use cairo::{self, Surface};
-use gdk_pixbuf;
-use gdk_sys;
 use glib::object::IsA;
 use glib::translate::*;
 use libc::{c_char, c_int};
 use std::ptr;
-use Cursor;
-use Visual;
-use Window;
 
-use {WindowHints, WindowType, WindowTypeHint, WindowWindowClass};
+use crate::{WindowType, WindowTypeHint, WindowWindowClass};
 
 pub struct WindowAttr {
     pub title: Option<String>,
-    pub event_mask: i32,
+    pub event_mask: EventMask,
     pub x: Option<i32>,
     pub y: Option<i32>,
     pub width: i32,
@@ -31,11 +28,11 @@ pub struct WindowAttr {
 }
 
 impl Default for WindowAttr {
-    fn default() -> WindowAttr {
+    fn default() -> Self {
         skip_assert_initialized!();
-        WindowAttr {
+        Self {
             title: None,
-            event_mask: 0,
+            event_mask: EventMask::empty(),
             x: None,
             y: None,
             width: 400,
@@ -51,62 +48,63 @@ impl Default for WindowAttr {
 }
 
 impl WindowAttr {
-    fn get_mask(&self) -> u32 {
-        let mut mask: gdk_sys::GdkWindowAttributesType = 0;
+    #[doc(alias = "get_mask")]
+    fn mask(&self) -> u32 {
+        let mut mask: ffi::GdkWindowAttributesType = 0;
         if self.title.is_some() {
-            mask |= gdk_sys::GDK_WA_TITLE;
+            mask |= ffi::GDK_WA_TITLE;
         }
         if self.x.is_some() {
-            mask |= gdk_sys::GDK_WA_X;
+            mask |= ffi::GDK_WA_X;
         }
         if self.y.is_some() {
-            mask |= gdk_sys::GDK_WA_Y;
+            mask |= ffi::GDK_WA_Y;
         }
         if self.cursor.is_some() {
-            mask |= gdk_sys::GDK_WA_CURSOR;
+            mask |= ffi::GDK_WA_CURSOR;
         }
         if self.visual.is_some() {
-            mask |= gdk_sys::GDK_WA_VISUAL;
+            mask |= ffi::GDK_WA_VISUAL;
         }
         if self.override_redirect {
-            mask |= gdk_sys::GDK_WA_NOREDIR;
+            mask |= ffi::GDK_WA_NOREDIR;
         }
         if self.type_hint.is_some() {
-            mask |= gdk_sys::GDK_WA_TYPE_HINT;
+            mask |= ffi::GDK_WA_TYPE_HINT;
         }
         mask
     }
 }
 
-#[cfg_attr(feature = "cargo-clippy", allow(type_complexity))]
-impl<'a> ToGlibPtr<'a, *mut gdk_sys::GdkWindowAttr> for WindowAttr {
+#[allow(clippy::type_complexity)]
+impl<'a> ToGlibPtr<'a, *mut ffi::GdkWindowAttr> for WindowAttr {
     type Storage = (
-        Box<gdk_sys::GdkWindowAttr>,
-        Stash<'a, *mut gdk_sys::GdkVisual, Option<Visual>>,
-        Stash<'a, *mut gdk_sys::GdkCursor, Option<Cursor>>,
+        Box<ffi::GdkWindowAttr>,
+        Stash<'a, *mut ffi::GdkVisual, Option<Visual>>,
+        Stash<'a, *mut ffi::GdkCursor, Option<Cursor>>,
         Stash<'a, *const c_char, Option<String>>,
     );
 
-    fn to_glib_none(&'a self) -> Stash<'a, *mut gdk_sys::GdkWindowAttr, Self> {
+    fn to_glib_none(&'a self) -> Stash<'a, *mut ffi::GdkWindowAttr, Self> {
         let title = self.title.to_glib_none();
         let visual = self.visual.to_glib_none();
         let cursor = self.cursor.to_glib_none();
 
-        let mut attrs = Box::new(gdk_sys::GdkWindowAttr {
+        let mut attrs = Box::new(ffi::GdkWindowAttr {
             title: title.0 as *mut c_char,
-            event_mask: self.event_mask,
+            event_mask: self.event_mask.bits() as i32,
             x: self.x.unwrap_or(0),
             y: self.y.unwrap_or(0),
             width: self.width,
             height: self.height,
-            wclass: self.wclass.to_glib(),
+            wclass: self.wclass.into_glib(),
             visual: visual.0,
-            window_type: self.window_type.to_glib(),
+            window_type: self.window_type.into_glib(),
             cursor: cursor.0,
             wmclass_name: ptr::null_mut(),
             wmclass_class: ptr::null_mut(),
-            override_redirect: self.override_redirect.to_glib(),
-            type_hint: self.type_hint.unwrap_or(WindowTypeHint::Normal).to_glib(),
+            override_redirect: self.override_redirect.into_glib(),
+            type_hint: self.type_hint.unwrap_or(WindowTypeHint::Normal).into_glib(),
         });
 
         Stash(&mut *attrs, (attrs, visual, cursor, title))
@@ -114,17 +112,19 @@ impl<'a> ToGlibPtr<'a, *mut gdk_sys::GdkWindowAttr> for WindowAttr {
 }
 
 impl Window {
+    #[doc(alias = "gdk_window_new")]
     pub fn new(parent: Option<&Window>, attributes: &WindowAttr) -> Window {
         assert_initialized_main_thread!();
         unsafe {
-            from_glib_full(gdk_sys::gdk_window_new(
+            from_glib_full(ffi::gdk_window_new(
                 parent.to_glib_none().0,
                 attributes.to_glib_none().0,
-                attributes.get_mask() as c_int,
+                attributes.mask() as c_int,
             ))
         }
     }
 
+    #[doc(alias = "gdk_window_create_similar_surface")]
     pub fn create_similar_surface(
         &self,
         content: cairo::Content,
@@ -132,7 +132,7 @@ impl Window {
         height: i32,
     ) -> Option<Surface> {
         unsafe {
-            from_glib_full(gdk_sys::gdk_window_create_similar_surface(
+            from_glib_full(ffi::gdk_window_create_similar_surface(
                 self.to_glib_none().0,
                 content.into(),
                 width,
@@ -140,69 +140,84 @@ impl Window {
             ))
         }
     }
+
+    #[doc(alias = "gdk_window_create_similar_image_surface")]
+    pub fn create_similar_image_surface(
+        &self,
+        format: cairo::Format,
+        width: i32,
+        height: i32,
+        scale: i32,
+    ) -> Option<cairo::Surface> {
+        unsafe {
+            from_glib_full(ffi::gdk_window_create_similar_image_surface(
+                self.to_glib_none().0,
+                format.into(),
+                width,
+                height,
+                scale,
+            ))
+        }
+    }
 }
 
 pub trait WindowExtManual: 'static {
+    #[doc(alias = "gdk_window_set_user_data")]
     unsafe fn set_user_data<T>(&self, user_data: &mut T);
 
-    #[cfg_attr(feature = "cargo-clippy", allow(mut_from_ref))]
-    unsafe fn get_user_data<T>(&self) -> &mut T;
+    #[allow(clippy::mut_from_ref)]
+    #[doc(alias = "gdk_window_get_user_data")]
+    #[doc(alias = "get_user_data")]
+    unsafe fn user_data<T>(&self) -> &mut T;
 
-    fn set_geometry_hints(&self, geometry: &gdk_sys::GdkGeometry, geom_mask: WindowHints);
-
-    fn get_default_root_window() -> Window;
+    #[doc(alias = "gdk_get_default_root_window")]
+    #[doc(alias = "get_default_root_window")]
+    fn default_root_window() -> Window;
 
+    #[doc(alias = "gdk_offscreen_window_set_embedder")]
     fn offscreen_window_set_embedder(&self, embedder: &Window);
 
+    #[doc(alias = "gdk_offscreen_window_get_embedder")]
     fn offscreen_window_get_embedder(&self) -> Option<Window>;
 
+    #[doc(alias = "gdk_offscreen_window_get_surface")]
     fn offscreen_window_get_surface(&self) -> Option<Surface>;
 
-    fn get_pixbuf(
-        &self,
-        src_x: i32,
-        src_y: i32,
-        width: i32,
-        height: i32,
-    ) -> Option<gdk_pixbuf::Pixbuf>;
+    #[doc(alias = "gdk_pixbuf_get_from_window")]
+    #[doc(alias = "get_pixbuf")]
+    fn pixbuf(&self, src_x: i32, src_y: i32, width: i32, height: i32)
+        -> Option<gdk_pixbuf::Pixbuf>;
 
-    fn get_background_pattern(&self) -> Option<cairo::Pattern>;
+    #[doc(alias = "gdk_window_get_background_pattern")]
+    #[doc(alias = "get_background_pattern")]
+    fn background_pattern(&self) -> Option<cairo::Pattern>;
 
+    #[doc(alias = "gdk_window_set_background_pattern")]
     fn set_background_pattern(&self, pattern: Option<&cairo::Pattern>);
 }
 
 impl<O: IsA<Window>> WindowExtManual for O {
     unsafe fn set_user_data<T>(&self, user_data: &mut T) {
-        gdk_sys::gdk_window_set_user_data(
+        ffi::gdk_window_set_user_data(
             self.as_ref().to_glib_none().0,
             user_data as *mut T as *mut _,
         )
     }
 
-    unsafe fn get_user_data<T>(&self) -> &mut T {
+    unsafe fn user_data<T>(&self) -> &mut T {
         let mut pointer = ::std::ptr::null_mut();
-        gdk_sys::gdk_window_get_user_data(self.as_ref().to_glib_none().0, &mut pointer);
+        ffi::gdk_window_get_user_data(self.as_ref().to_glib_none().0, &mut pointer);
         &mut *(pointer as *mut T)
     }
 
-    fn set_geometry_hints(&self, geometry: &gdk_sys::GdkGeometry, geom_mask: WindowHints) {
-        unsafe {
-            gdk_sys::gdk_window_set_geometry_hints(
-                self.as_ref().to_glib_none().0,
-                geometry,
-                geom_mask.to_glib(),
-            )
-        }
-    }
-
-    fn get_default_root_window() -> Window {
+    fn default_root_window() -> Window {
         assert_initialized_main_thread!();
-        unsafe { from_glib_none(gdk_sys::gdk_get_default_root_window()) }
+        unsafe { from_glib_none(ffi::gdk_get_default_root_window()) }
     }
 
     fn offscreen_window_set_embedder(&self, embedder: &Window) {
         unsafe {
-            gdk_sys::gdk_offscreen_window_set_embedder(
+            ffi::gdk_offscreen_window_set_embedder(
                 self.as_ref().to_glib_none().0,
                 embedder.to_glib_none().0,
             )
@@ -211,7 +226,7 @@ impl<O: IsA<Window>> WindowExtManual for O {
 
     fn offscreen_window_get_embedder(&self) -> Option<Window> {
         unsafe {
-            from_glib_none(gdk_sys::gdk_offscreen_window_get_embedder(
+            from_glib_none(ffi::gdk_offscreen_window_get_embedder(
                 self.as_ref().to_glib_none().0,
             ))
         }
@@ -220,13 +235,13 @@ impl<O: IsA<Window>> WindowExtManual for O {
     fn offscreen_window_get_surface(&self) -> Option<Surface> {
         skip_assert_initialized!();
         unsafe {
-            from_glib_none(gdk_sys::gdk_offscreen_window_get_surface(
+            from_glib_none(ffi::gdk_offscreen_window_get_surface(
                 self.as_ref().to_glib_none().0,
             ))
         }
     }
 
-    fn get_pixbuf(
+    fn pixbuf(
         &self,
         src_x: i32,
         src_y: i32,
@@ -235,7 +250,7 @@ impl<O: IsA<Window>> WindowExtManual for O {
     ) -> Option<gdk_pixbuf::Pixbuf> {
         skip_assert_initialized!();
         unsafe {
-            from_glib_full(gdk_sys::gdk_pixbuf_get_from_window(
+            from_glib_full(ffi::gdk_pixbuf_get_from_window(
                 self.as_ref().to_glib_none().0,
                 src_x,
                 src_y,
@@ -245,9 +260,9 @@ impl<O: IsA<Window>> WindowExtManual for O {
         }
     }
 
-    fn get_background_pattern(&self) -> Option<cairo::Pattern> {
+    fn background_pattern(&self) -> Option<cairo::Pattern> {
         unsafe {
-            let ret = gdk_sys::gdk_window_get_background_pattern(self.as_ref().to_glib_none().0);
+            let ret = ffi::gdk_window_get_background_pattern(self.as_ref().to_glib_none().0);
             if ret.is_null() {
                 None
             } else {
@@ -263,7 +278,7 @@ impl<O: IsA<Window>> WindowExtManual for O {
             } else {
                 ::std::ptr::null_mut()
             };
-            gdk_sys::gdk_window_set_background_pattern(self.as_ref().to_glib_none().0, ptr);
+            ffi::gdk_window_set_background_pattern(self.as_ref().to_glib_none().0, ptr);
         }
     }
 }
